#pragma language glsl3

#ifdef VERTEX
uniform mat4 MVP;
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform vec3 lightPosition;

attribute vec3 VertexNormal;

varying vec3 LightDirection_cameraspace;
varying vec3 Normal_cameraspace;
varying vec3 Position_worldspace;
varying vec3 EyeDirection_cameraspace;

vec4 position(mat4 transform_projection, vec4 vertex_position) {
	Position_worldspace = (modelMatrix * vertex_position).xyz;
	
	vec3 Position_cameraspace = (viewMatrix * modelMatrix * vertex_position).xyz; 
	EyeDirection_cameraspace = vec3(0,0,0) - Position_cameraspace;

	vec3 LightPosition_cameraspace = (viewMatrix * vec4(lightPosition,1)).xyz;
	LightDirection_cameraspace = LightPosition_cameraspace + EyeDirection_cameraspace;

	Normal_cameraspace = (viewMatrix * modelMatrix * vec4(VertexNormal,0)).xyz;
	return MVP * vertex_position;
}
#endif

#ifdef PIXEL
varying vec3 LightDirection_cameraspace;
varying vec3 Normal_cameraspace;
varying vec3 Position_worldspace;
varying vec3 EyeDirection_cameraspace;

uniform vec3 lightPosition;
uniform number lightPower;
uniform vec3 lightColor;

vec4 effect(vec4 color, Image texturee, vec2 texture_coords, vec2 screen_coords) {

	vec3 textureColor = Texel(texturee, texture_coords).rgb;
	textureColor = textureColor.rgb;
	vec3 textureAmbientColor = vec3(0.3,0.3,0.3)*textureColor;
	vec3 textureSpecularColor = vec3(0.3,0.3,0.3);


	vec3 n = normalize(Normal_cameraspace);
	vec3 l = normalize(LightDirection_cameraspace);

	number cosTheta = clamp(dot(n,  l), 0,1);
	number distance = length( lightPosition - Position_worldspace );

	vec3 E = normalize(EyeDirection_cameraspace);

	vec3 R = reflect(-l,n);

	number cosAlpha = clamp(dot(E,R), 0,1);


	

	return vec4((textureAmbientColor + 
			textureColor * lightColor * lightPower * cosTheta / (distance*distance) +
			textureSpecularColor * lightColor * lightPower * pow(cosAlpha,5) / (distance*distance)),1);
}
#endif
